/*! \file linked_list.h
    This file defines all data types used to store elements into the
    linked list.
 */

#ifndef LINKED_LIST_HEADER

  #include "grammar_types.h"
  #include "list.h"

  /*! \brief Defines the content of an element. An element may contain:
             <UL>
                <li>A word.
                <li>An operator.
             </UL>
   */

  enum expr_elem {
                   /*! \brief This type defines an element that contains a
                              word. */
                   word,

                   /*! \brief This type defines an element that contains an
                              operator. */
                   operator
                 };

  /*! \brief Defines the structure of an element of the linked list. */

  struct list_data {
                     /*! \brief If the element is a word, then this value points
                                to a zero terminated string of characters that
                                represents the word. Otherwise, it should be set
                                to the value NULL. */
                     char *word;

                     /*! \brief If the element is an operator, then this value
                                could be:
                                <UL>
                                  <li>or
                                  <li>and
                                  <li>xor
                                  <li>not
                                </UL>
                                Otherwise, it should be set to <i>undef</i>. */
                     enum operator_name operator;

                     /*! \brief Define the type of the element. If could be a
                                word or an opreator. */
                     enum expr_elem type;
                   };

  int add_element (
                    List *l,
                    enum expr_elem type,
                    enum operator_name op,
                    char *w
                 );
  void init_lined_list (List *l);
  void destroy_linked_list (List *l);

   /*! \brief This macro returns the type of data stored into an element.
       \param e Pointer to the element of the linked list.
       \return The macro returns one of the following values:
               <UL>
                 <li>word (see <i>enum expr_elem</i>)
                 <li>operator (see <i>enum expr_elem</i>)
               </UL>
     */

   #define GET_TYPE(e) (((struct list_data*)list_data(e))->type)

   /*! \brief This macro returns the word stored into an element.
       \param e Pointer to the element of the linked list.
       \return The macro returns a pointer to a zero terminated string of
               characters that represents the word.
       \warning You should call GET_TYPE(e) first.
   */

   #define GET_WORD(e) (((struct list_data*)list_data(e))->word)

   /*! \brief This macro returns the operator stored into an element.
       \param e Pointer to the element of the linked list.
       \return The macro returns one of the following values:
               <UL>
                 <li>or (see <i>enum operator_name</i>)
                 <li>and (see <i>enum operator_name</i>)
                 <li>xor (see <i>enum operator_name</i>)
                 <li>not (see <i>enum operator_name</i>)
                 <li>undef (see <i>enum operator_name</i>)
               </UL>
       \warning You should call GET_TYPE(e) first.
   */
   #define GET_OPERATOR(e) (((struct list_data*)list_data(e))->operator)

#define LINKED_LIST_HEADER
#endif


