/*! \file linked_list.c
    Linked list management for the boolean interpretor.
 */

#include <stdlib.h>
#include "linked_list.h"

/*! \brief User provided function used to free one element of the linked list.
    \param d Pointer to the element to free.
 */

static void free_data (void *d)
{
  struct list_data *data;

  data = (struct list_data*)d;
  if (data->type == word) { free (data->word); }

  free (data);
}

/*! \brief User provided function used to compare two elements of the linked
           list.
    \param d1 Pointer to the first element to compare.
    \warning This function is required by the linked list API. But it is not
             used by the interpretor.
    \param d2 Pointer to the second element to compare.
    \return The function returns 1 if the elements pointed by d1 and d2 are
            identical. It returns 0 otherwise.
 */

static int compare_element (const void* d1, const void* d2) { return 0; }

/*! \brief Initialize a given linked list so it can be used to save the result of the boolean analysis.
    \param l Pointer to the linked list to initialize.
 */

void init_lined_list (List *l)
{ list_init (l, free_data, compare_element); }

/*! \example test_bool.c */

/*! \brief Free all memory allocated for a linked list.
    \param l Pointer to the linked list to destroy.
 */

void destroy_linked_list (List *l)
{ list_destroy (l); }

/*! \example test_bool.c */

/*! \brief Add an element to the linked list.
    \param l Pointer to the linked list that will receive the new element.
    \param type Type of the new element. It could be:
           <UL>
              <li>word: for a word.
              <li>operator: for an operator.
           </UL>
    \param op If the value of <i>type</i> is <i>operator</i>, then the value
              of <i>op</i> could be:
              <UL>
                <li>and: for the operator <i>AND</i>.
                <li>or: for the operator <i>OR</i>.
              </UL>
              If the value of <i>type</i> is <i>word</i>, then the value
              of <i>op</i> should be set to <i>undef</i>.
    \param w If the value of <i>type</i> is <i>word</i>, then the pointer <i>
             w</i> should point to a zero terminated string of characters that
             represents a word. If the value of <i>type</i> is <i>operator</i>
             then the pointer <i>w</i> should be set to NULL.
    \return Upon sucessful completion the function returns 0, otherwise it
            returns 1 (this means that the program is running out of memory).
 */

int add_element (
                   List *l,
                   enum expr_elem type,
                   enum operator_name op,
                   char *w
                )
{
  struct list_data *d;

  /* --------------------------------------------------- */
  /* Allocate memory */
  /* --------------------------------------------------- */

  d = (struct list_data*)malloc(sizeof(struct list_data));
  if (d == NULL) { return 1; }

  /* --------------------------------------------------- */
  /* Initialize data */
  /* --------------------------------------------------- */

  d->type = type;

  switch (type)
  {
    case word: {
                 d->word = w;
                 d->operator = undef;
               }; break;

    case operator: {
                     d->word = NULL;
                     d->operator = op;
                   }; break;
  }

  /* --------------------------------------------------- */
  /* Insert data to the end of the linked list */
  /* --------------------------------------------------- */

  if (list_ins_next (l, list_tail(l), d) == -1)
  {
    free (d);
    return 1;
  }

  return 0;
}

/*! \example test_bool.c */


